TAG_AGENT = "proxy"
TAG_DIRECT = "direct"
TAG_BLOCKED = "block"

DNS_AGENT = "1.1.1.1"
DNS_DIRECT = "223.5.5.5"

HTTP_PROTOCOL = "http://"
HTTPS_PROTOCOL = "https://"

PREF_MODE = "pref_mode"
PREF_SPEED_ENABLED = True
PREF_SNIFFING_ENABLED = "pref_sniffing_enabled"
PREF_PROXY_SHARING = False
PREF_LOCAL_DNS_ENABLED = False
PREF_FAKE_DNS_ENABLED = "pref_fake_dns_enabled"
PREF_VPN_DNS = "pref_vpn_dns"
PREF_REMOTE_DNS = "pref_remote_dns"
PREF_DOMESTIC_DNS = "pref_domestic_dns"
PREF_LOCAL_DNS_PORT = "pref_local_dns_port"
PREF_ALLOW_INSECURE = "pref_allow_insecure"
PREF_SOCKS_PORT = "pref_socks_port"
PREF_HTTP_PORT = "pref_http_port"
PREF_LOGLEVEL = "info"
PREF_LANGUAGE = "pref_language"
PREF_PREFER_IPV6 = "pref_prefer_ipv6"
PREF_ROUTING_DOMAIN_STRATEGY = "pref_routing_domain_strategy"
PREF_ROUTING_MODE = "pref_routing_mode"
PREF_V2RAY_ROUTING_AGENT = "pref_v2ray_routing_agent"
PREF_V2RAY_ROUTING_DIRECT = "pref_v2ray_routing_direct"
PREF_V2RAY_ROUTING_BLOCKED = "pref_v2ray_routing_blocked"
PREF_PER_APP_PROXY = "pref_per_app_proxy"
PREF_PER_APP_PROXY_SET = "pref_per_app_proxy_set"
PREF_BYPASS_APPS = "pref_bypass_apps"
PREF_CONFIRM_REMOVE = "pref_confirm_remove"
PREF_START_SCAN_IMMEDIATE = "pref_start_scan_immediate"

PORT_LOCAL_DNS = 10853
PORT_SOCKS = 10808
PORT_HTTP = 10809
